//==============================================================
// Copyright  2020 Intel Corporation
//
// SPDX-License-Identifier: MIT
// =============================================================

#include "common.hpp"
#include "matrix_mul_rpi4b.hpp"

using namespace cl::sycl;

int main(int argc, char** argv)
{
	std::cout << "Parallel Matrix Multiplication on Raspberry Pi 4B+ (triSYCL/C++17) by Arthur V. Ratz @ CodeProject 2020\n";
	std::cout << "=======================================================================================================\n\n";

	// Declaring matrices A, B and C pointers
	short **A = nullptr, \
		**B = nullptr, **C = nullptr;

	// Generating random matrices A and B
	rand_matrix(A, N, M);
	rand_matrix(B, M, N);

	// Printing the elements of matrices A and B
	std::cout << "Matrix A:\n\n"; print_matrix(A, N, M);
	std::cout << "Matrix B:\n\n"; print_matrix(B, M, N);

	std::cout << "Multiplying matrices C = A x B:\n\n";

	// Obtaining the execution start time
	auto time_start = std::chrono::high_resolution_clock::now();

	// Executing the parallel code, computing the matrices A and B product
	mat_mul_sycl(A, B, C);

	// Obtaining the execution end time
	auto time_end = std::chrono::high_resolution_clock::now();

	// Printing all element of the resultant matrix C
	std::cout << "Matrix C:\n\n"; print_matrix(C, N, N);

	// Printing the duration of the parallel code execution in milliseconds
	std::cout << "\nExecution time: " << std::chrono::duration_cast< \
		std::chrono::milliseconds>(time_end - time_start).count() << " ms\n\n";

	return 0;
}